# ***********************************************************************************
#
# Lab12 "server" makefile
#
# This makefile is used to start the Codec Engine Server Wizard; then
# use it to build the DSP server package 
#
# ***********************************************************************************

# ---------------------------------------------------------------------
# setpaths.mak includes all absolute paths for DaVinci tools
#   and is located two levels above the /app directory.
# ---------------------------------------------------------------------
include ../../setpaths.mak


# *****************************************************************************
#
#    User-defined vars
#
# *****************************************************************************

# ---------------------------------------------------------------------
# AT: - Used for debug, it hides commands for a prettier output
#     - When debugging, you may want to set this variable to nothing by
#       setting it to "" below, or on the command line
# ---------------------------------------------------------------------
AT := @


#       --------------------------
# ------ Tools Location Variables --------------------------------------------
#       --------------------------
# These variables define where the various tools are located. In most cases
# they specify the "xs" (execute script) command found within the XDC tools;
# with the package to be executed is specified as its argument.
#
#(Note: the XDCPATH variable must include the package passed to 'xs' or
# 'xdc', as well as any package used during execution.)
#
# Short description of the tools:
# CDOC:       View documentation for RTSC packages along XDCPATH
# CONFIGURO:  Allows you to use packages as defined in a .cfg script.
# MAKEPKG:    This command runs the xdc tools.
# GENSERVER:  Kicks off the server creation wizard found in the codec engine.
CDOC      := $(XDC_INSTALL_DIR)/xs xdc.tools.cdoc.sq
CONFIGURO := $(XDC_INSTALL_DIR)/xs xdc.tools.configuro
GENSERVER := $(XDC_INSTALL_DIR)/xs ti.sdo.ce.wizards.genserver
MAKEPKG   := $(XDC_INSTALL_DIR)/xdc

#XDCARGS=CODEGEN_INSTALL_DIR=\"$(C6000_CG)\"

# The XDCPATH variable is the set of search directories used by the XDC
# tools (XS, XDC). It must contain the directory of each repository used by
# your program. (Repository is just a name for a folder that contains RTSC
# packages. Effectively, the combination of the repository - along with the
# package name - will provide the tools with the absolute path to your package.)
# For performance reasons, try to limit the number of directories included in
# the XDCPATH -- that is, less directories to search means the tools should
# run a bit faster. (Note, the XDCTOOLS location that xdc is being executed from
# is automatically appended to the XDCPATH.)
LAB12a_DIR     := /home/user/solutions/lab12a_build_server
LAB_DIR        := $(LAB12a_DIR)
export XDCPATH := $(LAB_DIR);$(CONFIG_BLD_PATH);$(CE_INSTALL_DIR)/packages;$(CE_INSTALL_DIR)/examples;$(CE_INSTALL_DIR)/cetools/packages;$(XDAIS_INSTALL_DIR)/packages;$(CMEM_INSTALL_DIR)/packages;$(FC_INSTALL_DIR)/packages;$(DSPLINK_INSTALL_DIR);$(BIOS_INSTALL_DIR)/packages;$(BIOSUTILS_INSTALL_DIR)/packages;$(XDCROOT)


# When running the XDC tool, it searches for a file called 'config.bld'. This file
# defines the build environment settings; that is, which targets and platforms are
# supported and characteristics about the targets (compiler options, tool locations,
# etc.).
#
# The XDCBUILDCFG variable is only needed when XDC is run; therefore, in the case
# of this makefile, it is only needed with the $(MAKEPKG) rule is executed.
#
# One caveat, you must make sure the 'config.bld' file specified by the XDCBUILDCFG
# variable is found on the XDCPATH (i.e. XDC search path). If you don't specify
# the XDCBUILDCFG variable, then XDC will use the first 'config.bld' isn't found
# file it finds along the XDCPATH search path. Likewise, if the file you specify
# in the XDCPATH, then again it looks for the first one it can find along
# the XDCPATH.
#
# Where does the 'config.bld' file come from? Besides writing one from scratch
# yourself, there are two choices:
# 1. A simple config.bld file was provided as part of the workshop files.
# 2. Another alternative is to use the 'config.bld' file provided in the Codec
#    Engine examples folder. This is a more complete (i.e. industrial strength)
#    version - but with that comes a bit more complexity. With the CE version,
#    you must update two files it depends on: user.bld, xdcpaths.mak.
#    The 'user.bld' file lets you specify the targets (and code gen tool locations)
#    that you want to build for. While 'xdcpaths.mak' provides a means to specify
#    the other tool locations - a duplicate, in some ways, to the tool paths
#    specified in this file. If you have already setup these two files, and/or
#    you want to build for multiple targets at a time, you may prefer to switch
#    to the CE examples 'config.bld' file. Both options are shown below, you just
#    need to comment one out, and uncomment the other.
#    (Note: in Codec Engine 2.24, they changed their method so that you only
#           need to specify the items you want to build, and tool locations, in
#           'xdcpaths.mak'. Also, it will inherit from the DVSDK Rules.mak, if
#           that file exists.
#
# Option 1 - Algorithm Wizard's simple config.bld
export XDCBUILDCFG := $(CONFIG_BLD_PATH)/config.bld

# Option 2 - Codec Engine example's config.bld (have you setup xdcpath.mak?)
  #export XDCBUILDCFG := $(CE_EXAMPLES)/config.bld

# Print the path to the config.bld to the command line while its running:
  $(warning Makefile specified this 'config.bld' location:  $(XDCBUILDCFG))

XDCARGS=CODEGEN_INSTALL_DIR=\"$(C6000_CG)\"

#uncomment this for verbose builds
#XDCOPTIONS=v

#       -----------------
# ------ DSP Server Rules --------------------------------------------------
#       -----------------

.PHONY: help
help: 
	@echo .
	@echo .---------------------------------------------------
	@echo . Run makefile with one of the following commands:
	@echo .   
	@echo .   make run_server_wizard
	@echo .   make build_server
	@echo .   make clean_server
	@echo .---------------------------------------------------
	@echo .
	
.PHONY: run_server_wizard run
run run_server_wizard:
	@echo .
	@echo . Running the server wizard ...
	$(GENSERVER)

.PHONY: build_server build
build build_server:
	@echo .
	@echo . Building server package ...
#	$(MAKEPKG) release -P .
	$(MAKEPKG) release XDCOPTIONS=$(XDCOPTIONS) XDCARGS="$(XDCARGS)" --xdcpath="$(XDCPATH)"
	@echo .

.PHONY: clean_server clean
clean clean_server:
	@echo .
	@echo . Cleaning server files ...
	$(MAKEPKG) clean -P .
	@echo .

